
tree<-tree[order(tree$unit,tree$plot,tree$tree),]
stand<-stand[order(stand$unit),]
trt<-trt[order(trt$unit,trt$stage),]

###########Generate treelist
######################################################################################################################
######################################################################################################################
out_standlist <- data.frame(Unit = double(), Cycle = integer(), Stage = integer(), TPA = double(), BA = double(), QMD = double(), Ht40=double(),CR = double(), RD = double(),
                           CFV = double(), BFV = double(), Thin = integer(), tCFV = double(), tBFV = double(), folret=double(),NPV = double())
out_treelist <- data.frame(Unit = double(), Cycle = integer(), Stage = integer(), Plot = integer(), Tree = integer(), Species = integer(), Dbh = double(),
                           Ht = double(), CR = double(), Expan = double(), Ht40=double(),CFV = double(), BFV = double(), Thin=integer(),
				   tCFV = double(),tBFV = double(),Thin_exp = double(),out_tr = integer(),merch_ht = double(),folret=double())

out_edit_error<-data.frame(Unit = double(), tree = integer(), error1=integer(), error2=integer())

nu=length(unique(stand$unit))#total number of individual units

## Create a dataframe to hold the total number of units for projection
test= matrix(ncol=2,nrow=(nu)) # Set matrix dimensions equal to the number of individual units
colnames(test) <- c("unit","groyrs") #

dfr= as.data.frame(test) # Convert to a data frame to facilitate referencing by column name
dfr[1:nu,]<-stand[c(1,2)]#

tcount=0#tree table adjuster for multiple unit runs
scount=0#stand table adjuster for multiple unit runs
tn=0


##############################################Load packages for extraction of soil, climate, and topographic data
###Determine if any spatial (lat,long) extractions will need to be made...if so, load packages 
spatial<-sum(stand$lat)
if(spatial !=0){
 if( "terra" %in% rownames(installed.packages()) == FALSE ) { install.packages("C:\\NEXANON\\R_packages\\terra_1.4-9.tar.gz",repos = NULL, type = "source") }
 if( "rgdal" %in% rownames(installed.packages()) == FALSE ) { install.packages("C:\\NEXANON\\R_packages\\rgdal_1.5-32.tar.gz",repos = NULL, type = "source") }
 if( "sp" %in% rownames(installed.packages()) == FALSE ) { install.packages("C:\\NEXANON\\R_packages\\sp_1.4-7.tar.gz",repos = NULL, type = "source") }
 if( "mapproj" %in% rownames(installed.packages()) == FALSE ) { install.packages("C:\\NEXANON\\R_packages\\mapproj_1.2.8.tar.gz",repos = NULL, type = "source") }

 require(terra)
 require(rgdal, warn.conflicts=FALSE)
 require(sp)
 require(mapproj)

 suppressMessages(require(terra,lib.loc=engine))
 suppressMessages(require(sp,lib.loc=engine))
 suppressMessages(require(rgdal,lib.loc=engine)) 
 suppressMessages(require(mapproj,lib.loc=engine))
}



############################################## Subset by unit
for(e in 1:nu) {

thinv3<- data.frame(Unit = double(), Cycle = integer(), Stage = integer(), Plot = integer(), Tree = integer(), Species = integer(), Dbh = double(),
                           Ht = double(), CR = double(), Expan = double(), Ht40=double(),CFV = double(), BFV = double(),
				   Thin = integer(),tCFV = double(),tBFV = double(),Thin_exp = double(),out_tr = integer(),merch_ht=double(),folret=double())


t1=0
t2=rep(0,2000)
ti=0
fi=0
TA1=0
tbc=0#count for BA thin based on density
trc=0#count for RD thin based on density
kill=0#error check for raster data
substand<-subset(stand, unit==dfr[e,1], select=unit:logtd) #subset the stand-level info by individual units
subtree<-subset(tree, unit==dfr[e,1], select=unit:expan) #subset the tree tree-level info by individual unit
if(length(subtree$unit)==0){
x=dfr[e,1]
stop("No tree records exist for stand ",x)
}

subtrt<-subset(trt, unit==dfr[e,1], select=unit:type) #subset the subtrt-level info by individual unit
substand$cfsh<-0
substand$cftd<-0
substand$logsh<-0.5
substand$logta<-8
substand$Max_SDI<-1
substand$DF_SDI<-0
substand$GW_DG<-0
substand$GW_HG<-0
substand$version<-3

#####Below handles empty treatment input
if(length((subtrt$unit))==0){
		names(subtrt)<-c("unit","stage","trt","value","trigger","sp_type","targ_sp","type")
		subtrt<-as.data.frame(subtrt)
		subtrt[1,1:8]<-cbind(substand[1,1],0,0,0,0,0,0,0)
}

n = length(subtree$tree)

# Initialize the CIPSEDIT vectors
SPECIES=rep(0,2000)
USER=rep(0,2000)
DBH=rep(0,2000)
HT=rep(0,2000)
CR=rep(0,2000)
EXPAN=rep(0,2000)
RADGRO=rep(0,2000)
RVARS=rep(0,30)
SERROR=rep(0,13)
TERROR=rep(0,2000*6)
SWARNING=rep(0,8)
TWARNING=rep(0,2000)
#DGRO=rep(0,2000)
GROWTH=rep(0,2000)
ACALIB=rep(1,3*18)
TREENO=rep(0,2000)
PTNO=rep(0,2000)
MGEXP=rep(0,2000)

VERSION = 3 # SMC variant
NPTS = length(unique(subtree$plot)) # Number of sample points fixed at one
NTREES = n # Number of trees in the 'big' sample, n is defined by the number above (length of input vector)
STAGE = substand$stage
BHAGE = substand$bhage # Breast height age
SPECIES[1:n] = subtree$species # Species for each tree
IEVEN = 1 # If the stand is even aged or not
DBH[1:n] = subtree$dbh # Diameter at breast height for each tree (in)
USER[1:n] = subtree$user # User code for thinning
HT[1:n] = subtree$ht # Total height for each tree (ft)
CR[1:n] = subtree$cr # Crown ratio for each tree 
EXPAN[1:n] = subtree$expan # Expansion factor for each tree (tpa)
RVARS[1]=substand$dfsi
RVARS[2]=substand$osi
RVARS[3:7] = c(substand$DF_SDI,0,0,0,0) # Set of seven indicator variables
RVARS[8]=0

IRAD = 0 # Indicator if radial growth measurements were entered

IERROR=0
TREENO[1:n]=subtree$tree # Create tree numbers if all set to zero					
PTNO[1:n] = subtree$plot # Sample number corresponding to each tree
MGEXP[1:n]=0


## Call the prepare subroutine in NEXANON_edit.dll to impute missing values in the sample
dyn.load("NEXANON_edit.dll", type="Fortran") 
prep = .Fortran("Prepare",
  as.integer(VERSION),
  as.integer(NPTS),
  as.integer(NTREES),
  as.integer(STAGE),
  as.integer(BHAGE),
  as.integer(SPECIES),
  as.integer(USER),
  as.integer(IEVEN),
  as.single(DBH),
  as.single(HT), 
  as.single(CR),
  as.single(EXPAN),
  as.single(RADGRO),
  as.single(RVARS),
  as.integer(SERROR), 
  as.integer(TERROR), 
  as.integer(SWARNING),
  as.integer(TWARNING),
  as.integer(IERROR),
  as.integer(IRAD),
  as.single(GROWTH),
  as.single(ACALIB)
) 

fatal = prep[[19]] # Check if a fatal error occured while processing the sample
  dyn.unload("NEXANON_edit.dll")


if(IERROR==0){#skip the projection on the substand if there are errors

## Format Cipsanon vectors for use in the CIPSRUN DLL (execute)
TREENO=rep(0,2000);PTNO=rep(0,2000);SPECIES=rep(0,2000);USER=rep(0,2000);INDS=rep(0,30);DBH1=rep(0,2000);HT1=rep(0,2000);
CR1=rep(0,2000);SCR1=rep(0,2000);EXPAN1=rep(0,2000);MGEXP=rep(0,2000);RVARS=rep(0,30);ACALIB=rep(0,3*18);PN=rep(0,25);
YSF=rep(0,25);BART=rep(0,25);YST=rep(0,25);NPR=rep(0,2000);PRAGE=rep(0,2000*3);PRLH=rep(0,2000*3);PRDBH=rep(0,2000*3);
PRHT=rep(0,2000*3);PRCR=rep(0,2000*3);PREXP=rep(0,2000*3);BRCNT=rep(0,2000*3);BRHT=rep(0,2000*40);BRDIA=rep(0,2000*40);
JCORE=rep(0,2000*40);SERROR=rep(0,40);TERROR=rep(0,2000*10);SWARNING=rep(0,9);TWARNING=rep(0,2000);DGRO=rep(0,2000);
HGRO=rep(0,2000);CRCHNG=rep(0,2000);SCRCHNG=rep(0,2000);MORTEXP=rep(0,2000);DBH2=rep(0,2000);HT2=rep(0,2000);
CR2=rep(0,2000);SCR2=rep(0,2000);EXPAN2=rep(0,2000);STOR=rep(0,30);rexpan=rep(0,2000);
  
## Supply arguments to each execute subroutine vector
VERSION = prep[[1]] # Version of Cipsanon to run
NPTS = prep[[2]] # Number of sample points fixed at one
NTREES1 = prep[[3]] # Number of trees prior to growing
STAGE = prep[[4]] # Stand age
BHAGE = prep[[5]] # Breast height age 
TREENO[1:n] = subtree$tree # Create tree numbers if all set to zero					
PTNO[1:n] = subtree$plot # Sample number corresponding to each tree
SPECIES = prep[[6]] # Species for each tree
USER = prep[[7]] # Code for user thinning

# Set of 16 indicator variables
INDS[1:30] = c(1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)

DBH1 = prep[[9]] # Diameter breast height for each tree (in), prior to growing   
HT1 = prep[[10]] # Total tree height for each tree (ft), prior to growing
CR1 = prep[[11]] # Crown ratio for each tree, prior to growing
SCR1 = prep[[11]] # Shadow crown ratio for each tree, prior to growing
EXPAN1 = prep[[12]] # Expansion factor for each tree (tpa), prior to growing

RVARS[1:30]=prep[[14]]
RVARS[1]=substand$dfsi
RVARS[2]=substand$osi
RVARS[3:5] = c(substand$DF_SDI,0,0) # Set of indicator variables
RVARS[6]=substand$GW_DG
RVARS[7]=substand$GW_HG
if((substand$GW_DG + substand$GW_HG)!=0){INDS[14]=1}
RVARS[8]=substand$DF_Folret
if((substand$lat!=0|substand$DF_Folret!=0) & substand$DF_Folret<3){INDS[15]=1}
if(substand$DF_Folret >= 3.0){RVARS[8]=0.0}
RVARS[9:30]=c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)

###################Extraction of spatial data
if(substand$lat!=0 & substand$DF_Folret==0){

	y<-substand$lat
	x<-substand$long

d <- data.frame(lon=x, lat=y)
coordinates(d) <- c("lon", "lat")
proj4string(d) <- CRS("+init=epsg:4326")

# transformed to fit in soil raster
d.new<-spTransform(d,CRS("+proj=aea +lat_0=23 +lon_0=-96 +lat_1=29.5 +lat_2=45.5 +x_0=0 +y_0=0 +datum=NAD83 +units=m +no_defs"))
c_s<-cbind(d.new$lon,d.new$lat)

# transformed to fit in topographic rasters
utm.new <- spTransform(d, CRS("+proj=utm +zone=10 +datum=WGS84"))
c_t<-cbind(utm.new$lon,utm.new$lat)

# not transformed for use in Climate NA rasters
c_c<-cbind(x,y)

soils_list<-read.csv("C:\\NEXANON\\Rasters\\soils_list2.csv", header = TRUE) 

  if(substand$DF_Folret==0){

	r_soil<-rast("C:\\NEXANON\\Rasters\\PNW_Westside.tif")
	mukey<-terra::extract(r_soil,c_s)
	colnames(mukey)<-c("mukey")

	r_tmax12<-rast("C:\\NEXANON\\Rasters\\Tmax12.tif")
	r_ppt05<-rast("C:\\NEXANON\\Rasters\\PPT05.tif")
	r_rh06<-rast("C:\\NEXANON\\Rasters\\RH06.tif")
	r_rh07<-rast("C:\\NEXANON\\Rasters\\RH07.tif")
	r_slope<-rast("C:\\NEXANON\\Rasters\\pnw30_slope.tif")
	r_aspect<-rast("C:\\NEXANON\\Rasters\\pnw30_aspect.tif")
	r_elev<-rast("C:\\NEXANON\\Rasters\\pnw_dem30.tif")
	r_tpi<-rast("C:\\NEXANON\\Rasters\\pnw_tpi200.tif")

	tmax12<-(terra::extract(r_tmax12,c_c))/10
	ppt05<-(terra::extract(r_ppt05,c_c))
	rh06<-(terra::extract(r_rh06,c_c))
	rh07<-(terra::extract(r_rh07,c_c))
	rh67<-(rh06+rh07)/2
	slope<-round((terra::extract(r_slope,c_c))/100000,1)
	aspect<-round((terra::extract(r_aspect,c_c)),1)
	tpi200<-(terra::extract(r_tpi,c_t))
	ma400<-(terra::extract(r_elev,c_c))-400
	if(is.nan(ma400$pnw_dem30)==TRUE){ma400$pnw_dem30<-0}
	if(ma400$pnw_dem30<0.0){ma400$pnw_dem30=0.0}
	rslope<-round(atan(slope/100),2)
	rasp<-round(aspect/57.2958,2)
	if(is.nan(ma400$pnw_dem30+tmax12$Tmax12+rasp$pnw30_aspect+rslope$pnw30_slope+rh67$RH06+tpi200$pnw_tpi200+ppt05$PPT05)==TRUE){
		rastout1<-paste("unit ", substand$unit, " is outside the range of the raster data", sep="")
		print(rastout1)
		kill=1
		}	
	mumerge<-merge(mukey, soils_list, by="mukey")
	if(length(mumerge$cec)!=0 & kill==0){
	RVARS[8]=4*(1-exp(-3.2698+0.0269*rh67+0.0266*tmax12+0.00144*ppt05+-0.136*rslope*cos(rasp)+-0.1261*rslope*sin(rasp)+-0.0337*ma400**0.5+0.00273*tpi200+0.0330*mumerge$cec**0.5))
	p=1
	}else{
	RVARS[8]=4*(1-exp(-3.2698+0.0269*rh67+0.0266*tmax12+0.00144*ppt05+-0.136*rslope*cos(rasp)+-0.1261*rslope*sin(rasp)+-0.0337*ma400**0.5+0.00273*tpi200+0.0330*(0.4685*y**0.5*exp(0.054*y)*exp(-0.2586*(125+x)))**0.5))
	p=2
	}
	if(kill==0){
		if(RVARS[8]>=3.0){
		RVARS[8]=3.0
	}
	}
  }	
  }# End of SNC extraction




ACALIB = prep[[22]] # prep calibration values
#######################################################
if(kill==0){
if(STAGE<17){
ACALIB = rep(1,3*18) # prep calibration values
}
########################################################
   PN = rep(0,5) # Number of pounds nitrogen applied; Most recent application at position one
   YSF = rep(0,5) # Years since fertilization; Most recent application year at position one 
   BABT = 0 # Basal area just before removal of trees
   BART = rep(0,5) # Basal area cut; Most recent cut at position one
   YST = rep(0,5) # Number of years since thinning; Most recent cut at position one
   NPR = rep(0,2000) # Number of prunings conducted for the ith tree
   PRAGE = rep(0,2000*3) # Age of pruning for the ith tree
   PRLH = rep(0,2000*3) # Lift height for the ith tree
   PRDBH = rep(0,2000*3) # DBH of the ith pruned tree
   PRHT =  rep(0,2000*3) # Total height of the ith pruned tree
   PRCR = rep(0,2000*3) # Crown ratio of the ith pruned tree
   PREXP = rep(0,2000*3) # Expansion factor for the ith pruned tree
   BRCNT = rep(0,2000*3) # Branch count 
   BRHT = rep(0,2000*40) # Height to branch
   BRDIA = rep(0,2000*40) # Branch diameter
   JCORE = rep(0,2000*40) # Juvenile wood core diameter
   SERROR = rep(0,35) # Array of stand errors 
   TERROR = rep(0,2000*6) # Array of tree errors
   SWARNING = rep(0,9) # Array of stand warnings
   TWARNING = rep(0,2000) # Array of tree warnings
   IERROR = 0 # Indicator for a fatal error
   NTREES2=0
   SNCHT1=rep(0,2000) 
   SNCHT2=rep(0,2000) 
   STOR = rep(0,30) # Array of variables; intialize to zero and found at first cycle call
  
# Supply arguments to each wood quality subroutine vector
IJCALC = 0 # Definition of the juvenile wood core
IFINAL = 0 # Indicator for a final felling
ACTION = 1 # Specify an initial fill in of wood quality values
NINGRO = 0 # Number of ingrowth trees
NWQT = 0 # Internal value of wood quality DLL
SITE_1 = substand$dfsi # Douglas-fir site index
SITE_2 = substand$osi # Other site index
PDEN = 0 # Starting number of Red Alder trees per acre
CYCLG=0

gy=substand$groyrs
#############################################Function for calculating dominant height (necessary for Random height growth effect)
h40f<-function(dbh, ht, expan) {
ns<-length(dbh)
a<-cbind(dbh, ht, expan)
b<-a[order(-(dbh)),]
b1<-cumsum(b[,3])
b2<-0
b3<-0
for(i in 1:ns){
if(b1[i]<=40){
b2[i]=b[i,3]
}
if(i>1){
if(b1[i]>40){
b2[i]=40-b1[i-1]
if(b1[i-1]>40){
b2[i]=0
}
}
}
b3[i]<-b[i,2]*b2[i]
}
hh40<-sum(b3)/40
return(hh40)
}
#############################################End of function for calculating dominant height 
if(CYCLG==0){
h40i<-h40f(prep[[9]][1:n],prep[[10]][1:n],prep[[12]][1:n])
}


## Prior to growing, create a blank matrix to store output from growing the stand
outtr= matrix(ncol=20,nrow=(4*n+n*gy)) # Set matrix dimensions equal to the maximum size possible under the projection length

## Specify the column names, and built a blank, 'big' tree list
colnames(outtr) <- c("Unit","Cycle","Stage","Plot","Tree","Species","Dbh","Ht","CR","Expan","Ht40","CFV","BFV","Thin","tCFV","tBFV","Thin_exp","out_tr","merch_ht","folret") #
outtr= as.data.frame(outtr) # Convert to a data frame to facilitate referencing by column name

## Insert the initial sample values prior to growing; Note that tree number is the user submitted identifier 
outtr[1:n,1:20] <- cbind(rep(substand[1,1],n),rep(0,n),rep(STAGE,n),PTNO[1:n],TREENO[1:n],SPECIES[1:n],DBH1[1:n],HT1[1:n],CR1[1:n],EXPAN1[1:n]/NPTS,rep(h40i,n),rep(0,n),rep(0,n),t2[1:n],rep(0,n),rep(0,n),rep(0,n),rep(substand$Tree_output,n),rep(0,n),rep(RVARS[[8]],n))#

t= length(subtrt[,1])		
##################################################################
#Adjust treatment requests into most appropriate 5-yr time period
for(i in 1:t) {
ptest<-(subtrt[i,2]-substand$stage)/5
rptest<-round((subtrt[i,2]-substand$stage)/5,digits=0)
if((rptest-ptest)!=0){
subtrt[i,2]<-substand$stage+(round((subtrt[i,2]-substand$stage)/5,digits=0)*5)
if(subtrt[i,2]<0){subtrt[i,2]=0}
}
}
##################################################################

frac<-(substand$groyrs/5)
gy<-ceiling(frac)


######################################### Grow the sample to user-specified rotation age
if(gy!=0){ ##differentiates between projecting and compiling current tree list
for(k in 1:gy) {


#scroll through treatments for application of fertilization
for(i in 1:t) {
######################################################################
##FERTILIZATION

if(substand[1,1]==subtrt[i,1]){
if(subtrt[i,3]==2){
if(subtrt[i,2]==STAGE){
fi=fi+1
INDS[8]=1 # indicator variable: stand has been fertilized
if(fi==1){

PN[1]=subtrt[i,4] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[1]=subtrt[i,2]-substand[1,3] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
}
if(fi==2){

PN[2]=PN[1] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[2]=YSF[1] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back

PN[1]=subtrt[i,4] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[1]=subtrt[i,2]-substand[1,3] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
}
if(fi==3){

PN[3]=PN[2] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[3]=YSF[2] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back

PN[2]=PN[1] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[2]=YSF[1] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back

PN[1]=subtrt[i,4] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[1]=subtrt[i,2]-substand[1,3] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
}
if(fi==4){

PN[4]=PN[3] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[4]=YSF[3] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back

PN[3]=PN[2] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[3]=YSF[2] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back

PN[2]=PN[1] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[2]=YSF[1] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back

PN[1]=subtrt[i,4] # lbs of N--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
YSF[1]=subtrt[i,2]-substand[1,3] # Years since fertilization--most recent fert is [1], previous is [2...]--subsequent ferts push each further back
}
}
}
}
}
##############################################################################################Test to make sure BHAGE remains 0 until dominant trees surpass BH
udbh<-data.frame(DBH1)
udbh<-udbh[1:n,1]
uexpan<-data.frame(prep[[12]])
if(CYCLG!=0){
uexpan<-data.frame(grow[[52]]/NPTS)
}
uexpan<-uexpan[1:n,1]

############################
for(i in 1:t) {#thinning treatment loop

#####################################################################
##Functions for use within thinning routines
##################################################################
utree<-data.frame(subtree$tree)
utree<-utree[1:n,1]
u<-cbind(utree, udbh, uexpan)
############################
ba_rd<-function(dbh, expan) {
ba<-((((dbh/24)**2)*3.1416)*expan)
rd<-(expan*(dbh/10)**1.605)/520
sumba<-sum(ba)
sumrd<-sum(rd)
sumc<-cbind(sumba,sumrd)
  return(sumc)
}
thintest<-ba_rd(udbh,uexpan)
############################
baf<-function(dbh, expan) {
ba<-((((dbh/24)**2)*3.1416)*expan)
sumba<-sum(ba)
return(sumba)
}
batest<-baf(udbh, uexpan)
############################
thinf<-function(tree, dbh, expan) {
ba<-((((dbh/24)**2)*3.1416)*expan)
rd<-(expan*(dbh/10)**1.605)/520
sort1<-cbind(tree, dbh, expan, ba, rd)
sort2<- sort1[order(-(dbh)),]
sba<-cumsum(sort2[,4])
srd<-cumsum(sort2[,5])
stpa<-cumsum(sort2[,3])
stt<-cbind(sort2, sba, srd, stpa)
return(stt)
}
if(ti==0){
thin<-thinf(utree,udbh,uexpan)
sort3<-u[order(-udbh),]
texpan<-sort3[,3]
rexpan<-sort3[,3]
thin1<-cbind(thin,texpan,rexpan)
thin2<-as.data.frame(thin1)
thin3 <- thin2[order(thin2$tree),] 
}
if(substand[1,1]==subtrt[i,1]){
if(subtrt[i,3]>2){
if(subtrt[i,2]==STAGE){
H40AT<-h40f(DBH1,HT1,EXPAN1)
}
}
######################################################################
##THIN by TPA##
if(subtrt[i,3]==5){ 
if(subtrt[i,2]==STAGE){

#initiate the vectors
SP=rep(0,2000)
D=rep(0,2000)
EXP=rep(0,2000)
MGE=rep(0,2000)
NPTS<-prep[[2]]
NTR<-prep[[3]]#number of trees in tree list
SP<-prep[[6]][1:NTR] # Species for each tree
D<-prep[[9]][1:NTR]
EXP<-prep[[12]][1:NTR] # Expansion factor for each tree (tpa)
if(CYCLG!=0){
NTR<-grow[[4]]#number of trees in tree list
NPTS<-grow[[3]]
SP<-grow[[9]][1:NTR] # Species for each tree
D<-grow[[48]][1:NTR] # Diameter at breast height for each tree (in)
EXP<-grow[[52]][1:NTR] # Expansion factor for each tree (tpa)
}
THINTYPE<-3 #TPA thin
SPTYP<-subtrt[i,6]# species type to thi
ISPEC<-subtrt[i,7] # species if targeting individual spec
TYP<-subtrt[i,8]# from above,below,proportional
RTPA<-subtrt[i,4] # residual TPA
TSDI<-0
RBA<-0
CODE<-0
USER<-rep(0,2000)
PCUT<-0
## Call the prepare subroutine in CIPSTHIN.dll to impute missing values in the sample
dyn.load("CIPSTHIN.dll", type="Fortran") 
tpathin = .Fortran("THIN",
  as.integer(THINTYPE),
  as.integer(NPTS),
  as.integer(SPTYP),
  as.integer(ISPEC),
  as.integer(TYP), 
  as.integer(SP),
  as.integer(NTR),
  as.single(D),
  as.single(EXP),
  as.single(RBA),
  as.single(TSDI),
  as.single(RTPA),
  as.single(CODE),
  as.single(USER),
  as.single(PCUT),
  as.single(MGE)
)
  dyn.unload("CIPSTHIN.dll") # Unload the execute subroutine  
bat<-baf(tpathin[[8]][1:NTR],tpathin[[9]][1:NTR])#BA before thinning
bar<-baf(tpathin[[8]][1:NTR],tpathin[[16]][1:NTR])#BA after thinning
for(a in 1:NTR) {
thin3[a,9]=tpathin[[9]][a]
thin3[a,10]=tpathin[[16]][a]
}
t1=1 #year of thinning indicator
t2=rep(1,2000)#stand has been thinned indicator

}
}
######################################################################
##THIN by USER 
if(subtrt[i,3]==6){ 
if(subtrt[i,2]==STAGE){


#initiate the vectors
USER=rep(0,2000)
EXP=rep(0,2000)
MGE=rep(0,2000)
NTR<-prep[[3]]#number of trees in tree list
USER<-prep[[7]][1:NTR] # Diameter at breast height for each tree (in)
NPTS<-prep[[2]]
EXP<-prep[[12]][1:NTR] # Expansion factor for each tree (tpa)
if(CYCLG!=0){
NPTS<-grow[[3]]
NTR<-grow[[4]]#number of trees in tree list
USER<-grow[[10]][1:NTR] # Diameter at breast height for each tree (in)
EXP<-grow[[52]][1:NTR] # Expansion factor for each tree (tpa)
}
CODE<-subtrt[i,5]
PCUT<-subtrt[i,4]#percentage of user code tree to be cut
THINTYPE<-4
RBA<-0
TSDI<-0
RTPA<-0
SPTYP<-0
ISPEC<-0
TYP<-0
SP<-prep[[6]][1:NTR] # species for each tree
D<-prep[[11]][1:NTR]

## Call the prepare subroutine in CIPSTHIN.dll to impute missing values in the sample
dyn.load("CIPSTHIN.DLL", type="Fortran") 
usthin = .Fortran("THIN",
  as.integer(THINTYPE),
  as.integer(NPTS),
  as.integer(SPTYP),
  as.integer(ISPEC),
  as.integer(TYP), 
  as.integer(SP),
  as.integer(NTR),
  as.single(D),
  as.single(EXP),
  as.single(RBA),
  as.single(TSDI),
  as.single(RTPA),
  as.single(CODE),
  as.single(USER),
  as.single(PCUT),
  as.single(MGE)
) 
  dyn.unload("CIPSTHIN.DLL") # Unload the execute subroutine  
if(k==1){
bat<-baf(prep[[11]][1:NTR],usthin[[9]][1:NTR])#BA before thinning
bar<-baf(prep[[11]][1:NTR],usthin[[16]][1:NTR])
}
if(k>1){
bat<-baf(grow[[12]][1:NTR],usthin[[9]][1:NTR])#BA before thinning
bar<-baf(grow[[12]][1:NTR],usthin[[16]][1:NTR])
}
for(a in 1:NTR) {
thin3[a,9]=usthin[[9]][a]
thin3[a,10]=usthin[[16]][a]
}

t1=1 #year of thinning indicator
t2=rep(1,2000)#stand has been thinned indicator

}
}
######################################################################
##THIN by SDI by density##
if(trc==0){
if(subtrt[i,3]==4){ 
if(subtrt[i,2]==0 & subtrt[i,5]<=thintest[1,2]*100){
NPTS<-grow[[3]]
NTR<-grow[[4]]#number of trees in tree list
#initiate the vectors
SP=rep(0,2000)
D=rep(0,2000)
EXP=rep(0,2000)
MGE=rep(0,2000)
SP<-grow[[9]][1:NTR] # Species for each tree
D<-grow[[48]][1:NTR] # Diameter at breast height for each tree (in)
EXP<-grow[[52]][1:NTR] # Expansion factor for each tree (tpa)

TYP<-subtrt[i,8]
# Maximum SDI for Red alder is 304: from Puettman, Hann, and Hibbs (1993)  
TSDI<-(subtrt[i,4]/100)*304
THINTYPE<-2
RBA<-0
RTPA<-0
CODE<-0
PCUT<-0
USER<-rep(0,2000)
SPTYP<-0
ISPEC<-0
## Call the prepare subroutine in CIPSTHIN.dll to impute missing values in the sample
dyn.load("CIPSTHIN.DLL", type="Fortran") 
sdithin = .Fortran("THIN",
  as.integer(THINTYPE),
  as.integer(NPTS),
  as.integer(SPTYP),
  as.integer(ISPEC),
  as.integer(TYP), 
  as.integer(SP),
  as.integer(NTR),
  as.single(D),
  as.single(EXP),
  as.single(RBA),
  as.single(TSDI),
  as.single(RTPA),
  as.single(CODE),
  as.single(USER),
  as.single(PCUT),
  as.single(MGE)
) 
  dyn.unload("CIPSTHIN.DLL") # Unload the execute subroutine  
bat<-baf(sdithin[[8]][1:NTR],sdithin[[9]][1:NTR])#BA before thinning
bar<-baf(sdithin[[8]][1:NTR],sdithin[[16]][1:NTR])
for(a in 1:NTR) {
thin3[a,9]=sdithin[[9]][a]
thin3[a,10]=sdithin[[16]][a]
}


t1=STAGE #year of thinning indicator
t2=rep(1,2000)#stand has been thinned indicator
trc=1
}
}
}
######################################################################
##THIN by SDI by age##
if(subtrt[i,3]==4){ 
if(subtrt[i,2]!=0 & subtrt[i,2]==STAGE){
#initiate the vectors
SP=rep(0,2000)
D=rep(0,2000)
EXP=rep(0,2000)
MGE=rep(0,2000)
NPTS<-prep[[2]]
NTR<-prep[[3]]#number of trees in tree list
SP<-prep[[6]][1:NTR] # Species for each tree
D<-prep[[9]][1:NTR]
EXP<-prep[[12]][1:NTR] # Expansion factor for each tree (tpa)
if(CYCLG!=0){
NPTS<-grow[[3]]
NTR<-grow[[4]]#number of trees in tree list
SP<-grow[[9]][1:NTR] # Species for each tree
D<-grow[[48]][1:NTR] # Diameter at breast height for each tree (in)
EXP<-grow[[52]][1:NTR] # Expansion factor for each tree (tpa)
}
TYP<-subtrt[i,8]
# Maximum SDI for Red alder is 304: from Puettman, Hann, and Hibbs (1993) 
TSDI<-(subtrt[i,4]/100)*304
THINTYPE<-2
RBA<-0
RTPA<-0
CODE<-0
PCUT<-0
USER<-rep(0,2000)
SPTYP<-0
ISPEC<-0
## Call the prepare subroutine in CIPSTHIN.dll to impute missing values in the sample
dyn.load("CIPSTHIN.DLL", type="Fortran") 
sdithin = .Fortran("THIN",
  as.integer(THINTYPE),
  as.integer(NPTS),
  as.integer(SPTYP),
  as.integer(ISPEC),
  as.integer(TYP), 
  as.integer(SP),
  as.integer(NTR),
  as.single(D),
  as.single(EXP),
  as.single(RBA),
  as.single(TSDI),
  as.single(RTPA),
  as.single(CODE),
  as.single(USER),
  as.single(PCUT),
  as.single(MGE)
) 
  dyn.unload("CIPSTHIN.DLL") # Unload the execute subroutine  
bat<-baf(sdithin[[8]][1:NTR],sdithin[[9]][1:NTR])#BA before thinning
bar<-baf(sdithin[[8]][1:NTR],sdithin[[16]][1:NTR])
for(a in 1:NTR) {
thin3[a,9]=sdithin[[9]][a]
thin3[a,10]=sdithin[[16]][a]
}


t1=1 #year of thinning indicator
t2=rep(1,2000)#stand has been thinned indicator
}
}
######################################################################
##THIN by BA by density##
if(tbc==0){
if(subtrt[i,3]==3){ 
if(subtrt[i,2]==0 & subtrt[i,5]<=thintest[1,1]){

NPTS<-grow[[3]]
NTR<-grow[[4]]#number of trees in tree list
#initiate the vectors
SP=rep(0,2000)
D=rep(0,2000)
EXP=rep(0,2000)
MGE=rep(0,2000)

SP<-grow[[9]][1:NTR] # Species for each tree
D<-grow[[48]][1:NTR] # Diameter at breast height for each tree (in)
EXP<-grow[[52]][1:NTR] # Expansion factor for each tree (tpa)

SPTYP<-subtrt[i,6]
ISPEC<-subtrt[i,7]
TYP<-subtrt[i,8]
RBA<-subtrt[i,4]
THINTYPE<-1
TSDI<-0
RTPA<-0
PCUT<-0
CODE<-0
USER=rep(0,2000)

## Call the prepare subroutine in CIPSTHIN.dll to impute missing values in the sample
dyn.load("CIPSTHIN.DLL", type="Fortran") 
bathin = .Fortran("THIN",
  as.integer(THINTYPE),
  as.integer(NPTS),
  as.integer(SPTYP),
  as.integer(ISPEC),
  as.integer(TYP), 
  as.integer(SP),
  as.integer(NTR),
  as.single(D),
  as.single(EXP),
  as.single(RBA),
  as.single(TSDI),
  as.single(RTPA),
  as.single(CODE),
  as.single(USER),
  as.single(PCUT),
  as.single(MGE)
) 
  dyn.unload("CIPSTHIN.DLL") # Unload the execute subroutine  
bat<-baf(bathin[[8]][1:NTR],bathin[[9]][1:NTR])#BA before thinning
bar<-baf(bathin[[8]][1:NTR],bathin[[16]][1:NTR])
for(a in 1:NTR) {
thin3[a,9]=bathin[[9]][a]
thin3[a,10]=bathin[[16]][a]
}

t1=STAGE #year of thinning indicator
t2=rep(1,2000)#stand has been thinned indicator
tbc=1
}
}
}

#########################################################################
######################################################################
##THIN by BA by age##

if(subtrt[i,3]==3){ 
if(subtrt[i,2]!=0 & subtrt[i,2]==STAGE){
#initiate the vectors
SP=rep(0,2000)
D=rep(0,2000)
EXP=rep(0,2000)
MGE=rep(0,2000)
NPTS<-prep[[2]]
NTR<-prep[[3]]#number of trees in tree list
SP<-prep[[6]][1:NTR] # Species for each tree
D<-prep[[9]][1:NTR]
EXP<-prep[[12]][1:NTR] # Expansion factor for each tree (tpa)
if(CYCLG!=0){
NPTS<-grow[[3]]
NTR<-grow[[4]]#number of trees in tree list
SP<-grow[[9]][1:NTR] # Species for each tree
D<-grow[[48]][1:NTR] # Diameter at breast height for each tree (in)
EXP<-grow[[52]][1:NTR] # Expansion factor for each tree (tpa)
}
SPTYP<-subtrt[i,6]
ISPEC<-subtrt[i,7]
TYP<-subtrt[i,8]
RBA<-subtrt[i,4]
THINTYPE<-1
TSDI<-0
RTPA<-0
PCUT<-0
CODE<-0
USER=rep(0,2000)
## Call the prepare subroutine in CIPSTHIN.dll to impute missing values in the sample
dyn.load("CIPSTHIN.DLL", type="Fortran") 
bathin = .Fortran("THIN",
  as.integer(THINTYPE),
  as.integer(NPTS),
  as.integer(SPTYP),
  as.integer(ISPEC),
  as.integer(TYP), 
  as.integer(SP),
  as.integer(NTR),
  as.single(D),
  as.single(EXP),
  as.single(RBA),
  as.single(TSDI),
  as.single(RTPA),
  as.single(CODE),
  as.single(USER),
  as.single(PCUT),
  as.single(MGE)
) 
  dyn.unload("CIPSTHIN.DLL") # Unload the execute subroutine  
bat<-baf(bathin[[8]][1:NTR],bathin[[9]][1:NTR])#BA before thinning
bar<-baf(bathin[[8]][1:NTR],bathin[[16]][1:NTR])
for(a in 1:NTR) {
thin3[a,9]=bathin[[9]][a]
thin3[a,10]=bathin[[16]][a]
}
t1=1 #year of thinning indicator
t2=rep(1,2000)#stand has been thinned indicator
}
}
}


#########################################################################

##Join expansion factors to rest of the dataset

if(t1!=0){
INDS[7]=1 # indicator variable: stand has been partially cut
INDS[13]=1 # indicator variable: major conifer trees have been removed
ti=ti+1 #counter to track total number of thins
ptexpf<-subset(thin3, select=c(9,10))##keep the post thinning tree numbers and new expansion factors#EXPAN1[1:n]<-ptexpf[1:n,1]

###################################################################################################
#####need all of the following variables to be the same length(1:n)
thinv<-cbind(rep(substand[1,1],n),rep(CYCLG,n),PTNO[1:n],TREENO[1:n],SPECIES[1:n],DBH1[1:n],HT1[1:n],CR1[1:n],ptexpf[1:n,2],rep(0,n),rep(0,n),rep(0,n))

thinv1= matrix(ncol=12,nrow=(n)) # Set matrix dimensions equal to the maximum size possible under the projection length
thinv1= as.data.frame(thinv) # Convert to a data frame to facilitate referencing by column name
colnames(thinv1) <- c("Unit","Period","Plot","tree","Species","Dbh","Ht","CR","Thin_exp","Ht40","CFV","BFV") 


if(ti==1){
if(t1|subtrt[i,2]==STAGE){
BART[1]=bar # BA removed--most recent thin is [1], previous is [2...]--subsequent thins push each further back
BABT=bat # Initial pre-thin BA
YST[1]=subtrt[i,2]-substand[1,3] # Years since thinning--most recent fert is [1], previous is [2...]--subsequent thins push each further back
if(t1==STAGE){YST[1]=t1-substand[1,3]} #  Years since start of run that thin occured--most recent thin is [1], previous is [2...]--subsequent thins push each further back
TA1=(subtrt[i,2])#thinning age
if(subtrt[i,2]==0){
TA1=t1
}
}
}
if(ti==2){
if(t1|subtrt[i,2]==STAGE){
TA2=TA1 #thinning age
BART[2]=BART[1] # BA removed--most recent thin is [1], previous is [2...]--subsequent thins push each further back
YST[2]=YST[1] # Years since thinning--most recent fert is [1], previous is [2...]--subsequent thins push each further back
TA1=(subtrt[i,2])#Thinning age
if(subtrt[i,2]==0){
TA1=t1
}
BART[1]=bar # BA removed--most recent thin is [1], previous is [2...]--subsequent thins push each further back
BABT=bat # Initial pre-thin BA
YST[1]=subtrt[i,2]-substand[1,3] # Years since thinning--most recent fert is [1], previous is [2...]--subsequent thins push each further back
if(t1==STAGE){YST[1]=t1-substand[1,3]} #  Years since start of run that thin occured--most recent thin is [1], previous is [2...]--subsequent thins push each further back
}
}
if(ti==3){
if(t1|subtrt[i,2]==STAGE){
TA3=TA2 #thinning age
BART[3]=BART[2] # BA removed--most recent thin is [1], previous is [2...]--subsequent thins push each further back
YST[3]=YST[2] # Years since thinning--most recent fert is [1], previous is [2...]--subsequent thins push each further back
TA2=TA1#thinning age
BART[2]=BART[1] # BA removed--most recent thin is [1], previous is [2...]--subsequent thins push each further back
YST[2]=YST[1] # Years since thinning--most recent fert is [1], previous is [2...]--subsequent thins push each further back
TA1=(subtrt[i,2])#thinning age
if(subtrt[i,2]==0){
TA1=t1
}
BART[1]=bar # BA removed--most recent thin is [1], previous is [2...]--subsequent thins push each further back
BABT=bat # Initial pre-thin BA
YST[1]=subtrt[i,2]-substand[1,3] # Years since thinning--most recent fert is [1], previous is [2...]--subsequent thins push each further back
if(t1==STAGE){YST[1]=t1-substand[1,3]} #  Years since start of run that thin occured--most recent thin is [1], previous is [2...]--subsequent thins push each further back
}
}
t1=0

#This is used to calculate the thinning expansion factors for stand level output
MGEXP[1:n]<-thinv1$Thin_exp   #expansion factors of removed trees
EXPAN1[1:n]<-(EXPAN1[1:n]-thinv1$Thin_exp)   #post-thin expansion factors
h40t<-h40f(thinv1$Dbh,thinv1$Ht,EXPAN1[1:n]/NPTS)
thinv2<-cbind(thinv1[,1],rep((k-1),n),rep(STAGE,n),thinv1[,3:8],EXPAN1[1:n]/NPTS,rep(h40t,n),rep(0,n),rep(0,n),rep(1,n),rep(0,n),rep(0,n),MGEXP[1:n]/NPTS,rep(substand$Tree_output,n),rep(0,n),rep(RVARS[8],n))
colnames(thinv2) <- c("Unit","Cycle","Stage","Plot","Tree","Species","Dbh","Ht","CR","Expan","Ht40","CFV","BFV","Thin","tCFV","tBFV","Thin_exp","out_tr","merch_ht","folret") #
thinv3<-rbind(thinv3,thinv2)

t2=rep(0,2000)

}
}#end of treatment tests


######################################################################
##################################################################

##This is loop to apply treatments to projection

  ## Call the execute subroutine in CIPSRUN.dll to grow the sample
  dyn.load("NEXANON_run.dll", type="Fortran") # Load the execute subroutine 
  grow = .Fortran("execute",
    as.integer(CYCLG),
    as.integer(VERSION),
    as.integer(NPTS), 
    as.integer(NTREES1), 
    as.integer(STAGE), 
    as.integer(BHAGE), 
    as.integer(TREENO), 
    as.integer(PTNO), 
    as.integer(SPECIES), 
    as.integer(USER),
    as.integer(INDS), 
    as.single(DBH1), 
    as.single(HT1), 
    as.single(CR1), 
    as.single(SCR1), 
    as.single(EXPAN1), 
    as.single(MGEXP),
    as.single(RVARS), 
    as.single(ACALIB), 
    as.single(PN), 
    as.single(YSF),
    as.single(BABT), 
    as.single(BART), 
    as.single(YST),
    as.integer(NPR),
    as.integer(PRAGE), 
    as.single(PRLH), 
    as.single(PRDBH), 
    as.single(PRHT), 
    as.single(PRCR), 
    as.single(PREXP), 
    as.integer(BRCNT), 
    as.integer(BRHT),
    as.integer(BRDIA), 
    as.integer(JCORE), 
    as.integer(SERROR), 
    as.integer(TERROR), 
    as.integer(SWARNING), 
    as.integer(TWARNING), 
    as.integer(IERROR),
    as.single(DGRO), 
    as.single(HGRO), 
    as.single(CRCHNG), 
    as.single(SCRCHNG), 
    as.single(MORTEXP), 
    as.integer(NTREES2), 
    as.single(DBH2), 
    as.single(HT2),
    as.single(CR2), 
    as.single(SCR2), 
    as.single(EXPAN2),
    as.single(SNCHT1), 
    as.single(SNCHT2),
    as.single(STOR)
  ) 
 
  dyn.unload("NEXANON_run.dll") # Unload the execute subroutine 

  ## Update the  dyn.unload("cipsrun.DLL") # Unload the execute subroutine values for the grown sample
  CYCLG = k # Update the value of cycles grown so far
  TREENO = grow[[7]] # Tree numbers
  PTNO = grow[[8]] # Sample number
  SPECIES = grow[[9]] # Tree species
  USER = grow[[10]] # Update the user code
  STAGE = grow[[5]] # Stand age
  BHAGE = grow[[6]] # Breast height age
############################Function to make sure BHAGE remains 0 until dominant trees surpass BH
#if(dt<40){
#BHAGE<-0
#}
############################

	D=grow[[47]]
	H=grow[[48]]
	E=grow[[51]]
	if(INDS[15]==1){
	H=grow[[53]]
	}

	ht40<-h40f(D,H,E)#calculate new Ht40 with Ht40 function
	###Determine random height increment variation to be added to the final estimate of height

	DBH1=grow[[47]]		
	HT1=grow[[48]]
      SNCHT1=grow[[53]][1:NTREES1]
	EXPAN1=grow[[51]]
	CR1=grow[[49]]
	SCR1=grow[[50]]

if(gy==k){
if(frac-floor(frac)>0){
	fr<-(frac-floor(frac))
	DBH1=grow[[12]]+(grow[[47]]-grow[[12]])*fr		
	HT1=grow[[13]]+(grow[[48]]-grow[[13]])*fr
###Below line could be a problem
	SNCHT1=grow[[53]]-grow[[42]]*(1-fr)*(1-exp(-1.1826-0.107*substand$DF_Folret**3))
	EXPAN1=grow[[16]]+(grow[[51]]-grow[[16]])*fr
	CR1=grow[[14]]+(grow[[49]]-grow[[14]])*fr
	SCR1=grow[[15]]+(grow[[50]]-grow[[15]])*fr
	STAGE=STAGE-5*(1-fr)
	BHAGE=BHAGE-5*(1-fr)
      ht40<-h40f(DBH1,HT1,EXPAN1)#calculate new Ht40 with Ht40 function
	if(INDS[15]==1){
      ht40<-h40f(DBH1,SNCHT1,EXPAN1)#calculate new Ht40 with Ht40 function
      }
}
}



# Insert the updated tree values into the executed value table; As before, the user submitted tree number is assigned
if(INDS[15]==0){			
outtr[(k*n+1+tcount):((k*n+1)+n-1+tcount),1:20] <-cbind(rep(substand$unit,n),rep(k,n),rep(STAGE,n),PTNO[1:n],TREENO[1:n],SPECIES[1:n],DBH1[1:n],HT1[1:n],CR1[1:n],EXPAN1[1:n]/NPTS,ht40,rep(0,n),rep(0,n),t2[1:n],rep(0,n),rep(0,n),rep(0,n),rep(substand$Tree_output,n),rep(0,n),rep(RVARS[[8]],n))#
}
if(INDS[15]==1){	
outtr[(k*n+1+tcount):((k*n+1)+n-1+tcount),1:20] <-cbind(rep(substand$unit,n),rep(k,n),rep(STAGE,n),PTNO[1:n],TREENO[1:n],SPECIES[1:n],DBH1[1:n],SNCHT1[1:n],CR1[1:n],EXPAN1[1:n]/NPTS,ht40,rep(0,n),rep(0,n),t2[1:n],rep(0,n),rep(0,n),rep(0,n),rep(substand$Tree_output,n),rep(0,n),rep(RVARS[[8]],n))#
}
} #End of growth loop
if(gy==0){
outtr[1:n,1:20] <-cbind(rep(substand$unit,n),rep(k,n),rep(STAGE,n),PTNO[1:n],TREENO[1:n],SPECIES[1:n],DBH1[1:n],HT1[1:n],CR1[1:n],EXPAN1[1:n]/NPTS,rep(h40i,n),rep(0,n),rep(0,n),t2[1:n],rep(0,n),rep(0,n),rep(0,n),rep(substand$Tree_output,n),rep(0,n),rep(RVARS[[8]],n))#
}
} #End of projection/compile test



outtrx<-rbind(outtr,thinv3)###add in data from thinning
outtrx<- outtrx[!is.na(outtrx$Species),]#gets rid of rows without records
outtrx<-outtrx[order(outtrx$Unit,outtrx$Cycle,outtrx$Plot,outtrx$Thin,outtrx$Tree),]
outtr1<-subset(outtrx, Species!=0)#get rid of rows with species = 0 (no records)

#remove thinning data frames
if(ti!=0){
rm("thinv","thinv1","thinv2")
}

##################################################################################################
###Volume output
##################################################################################################

			## Calculate volume for each unthinned tree
			TERROR=rep(0,2000*4);TWARNING=rep(0,2000*4);VERROR=rep(0,5);VWARNING=rep(0,5);CFVOL=rep(0,2000);BFVOL=rep(0,2000);
			VERSION=substand$version;MH=rep(0,2000);
			## Supply arguments to each volume subroutine vector
			CFTD = substand$cftd # Top diameter inside bark for cubic foot volume
			CFSH = substand$cfsh # Stump height for cubic foot volume 
			LOGLL = substand$logll # Log length for Scribner volume
			LOGML = substand$logml # Minimum log length for Scribner volume
			LOGTD = substand$logtd # Top diameter inside bark for Scriber volume
			LOGSH = substand$logsh # Stump height for Scribner volume
			LOGTA = substand$logta # Trim allowance for Scribner volume
			DFFR = substand$DF_Folret # Douglas-fir foliage retention for SNC
			if(substand$DF_Folret >= 3.0){DFFR=0}
			## Estimate volume for the sampled trees
			dyn.load("CIPSVOL_SNC.dll",type="Fortran") # Load the volume subroutine
## Depending on tree list size, simply process or split into equal intervals and process
			if(nrow(outtr1)<2000){			
			## Depending on tree list size, simply process or split into equal intervals and process
				## Calculate volume estimates for each tree
				NTREES=nrow(outtr1) # Number of trees to process
				UNIT<-dfr[e,1]
				SPECIES <- outtr1$Species[1:NTREES] # Tree species
				DBH <- outtr1$Dbh[1:NTREES] # Diameter at breast height (in)
				D15<-rep(0,NTREES)#D15 is zero due to no D15 measurements
				HT <- outtr1$Ht[1:NTREES] # Total tree height (ft)
				CR <- outtr1$CR[1:NTREES] # Crown ratio 
				volume = .Fortran("volcal",as.integer(VERSION),as.integer(NTREES),as.integer(SPECIES),as.single(CFTD),as.single(CFSH),as.single(LOGLL),as.single(LOGML),as.single(LOGTD), 
						as.single(LOGSH),as.single(LOGTA),as.single(DBH),as.single(D15),as.single(HT),as.single(CR),as.single(DFFR),as.integer(VERROR),as.integer(TERROR),as.integer(VWARNING), 
						as.integer(TWARNING),as.integer(IERROR),as.single(CFVOL),as.single(BFVOL))
				outtr1$CFV = volume[[21]][1:NTREES] # Input the cubic foot volume for each tree
				outtr1$BFV = volume[[22]][1:NTREES] # Input the board foot volume for each tree
			} else {
				int = findInterval(1:nrow(outtr1),seq(2001,2001*CYCLG,by=2000)) # Find splitting intervals [every 2000]
				outtr1=split(outtr1,int) # Split up the tree list by the found interval(s)
				outtr1 = lapply(outtr1,function(df){
				## Calculate volume estimates for each tree
				NTREES=nrow(df) # Number of trees to process
				UNIT <-(dfr[e])
				SPECIES <- df$Species[1:NTREES] # Tree species
				DBH <- df$Dbh[1:NTREES] # Diameter at breast height (in)
				D15<-rep(0,NTREES)#D15 is zero due to no D15 measurements
				HT <- df$Ht[1:NTREES] # Total tree height (ft)
				CR <- df$CR[1:NTREES] # Crown ratio 
				volume = .Fortran("volcal",as.integer(VERSION),as.integer(NTREES),as.integer(SPECIES),as.single(CFTD),as.single(CFSH),as.single(LOGLL),as.single(LOGML),as.single(LOGTD), 
						as.single(LOGSH),as.single(LOGTA),as.single(DBH),as.single(D15),as.single(HT),as.single(CR),as.single(DFFR),as.integer(VERROR),as.integer(TERROR),as.integer(VWARNING), 
						as.integer(TWARNING),as.integer(IERROR),as.single(CFVOL),as.single(BFVOL))
				df$CFV = volume[[21]][1:NTREES] # Input the cubic foot volume for each tree
				df$BFV = volume[[22]][1:NTREES] # Input the board foot volume for each tree
				df # Return the processed (cut section) data frame
							
						}
				)
				outtr1 = unsplit(outtr1,int) # Return to native format

			}						
			dyn.unload("CIPSVOL_SNC.dll") # Unload the volume calculation DLL
			

##################################################################################################
###end of volume output
##################################################################################################
	

	outtr1$tCFV[is.na(outtr1$tCFV)]<-0.0 #convert "NA" to zero
	outtr1$tBFV[is.na(outtr1$tBFV)]<-0.0 #convert "NA" to zero
	#calculate thinned volume and join with main dataset before conglomerating
	if(ti>0){
		outtr1$tCFV<-round((outtr1$Thin*outtr1$CFV*(outtr1$Thin_exp/outtr1$Thin_exp)),1)#calculate thinned per acre cubic volume per tree
		outtr1$tBFV<-round((outtr1$Thin*outtr1$BFV*(outtr1$Thin_exp/outtr1$Thin_exp)),1)#calculate thinned per acre board foot volume per tree
		outtr1$Thin_exp[is.na(outtr1$Thin_exp)]<-0 #convert "NA" to zero
		}
###Zero out volumes for pre-commercially thinned stands
	trco<-length(outtr1$Stage)
	for(i in 1:t) {
	if(substand[1,1]==subtrt[i,1]){
	if(subtrt[i,3]==5){
	if(subtrt[i,2]<11){
	for(j in 1:trco) {	
	if(outtr1[j,3]<8){
	outtr1[j,15]<-0
	outtr1[j,16]<-0
	}
	}
	}
	}
	}
	}

	outtr1$tCFV[is.na(outtr1$tCFV)]<-0.0 #convert "NA" to zero
	outtr1$tBFV[is.na(outtr1$tBFV)]<-0.0 #convert "NA" to zero
	outtr1$Thin_exp[is.na(outtr1$Thin_exp)]<-0.0 #convert "NA" to zero




#accumulate output from multiple units
	if(nu>1){
		out_treelist <- rbind(out_treelist, outtr1)
		}else{
			out_treelist <- (outtr1)
	}

#Conglomerate tree level values 
	if(ti>0){
##reset thinning expansion factors to zero
		for(a in 1:n) {
		ptexpf[a,]<-0
		}
	}	


}#end of site-level kill skip loop
}#end of IERROR=1 check
}#end of subsetting loop

#round tree-level variables
out_treelist$Dbh<-round(out_treelist$Dbh,3)
out_treelist$Ht<-round(out_treelist$Ht,1)
out_treelist$CR<-round(out_treelist$CR,2)
out_treelist$Expan<-round(out_treelist$Expan,3)
out_treelist$CFV<-round(out_treelist$CFV,1)
out_treelist$BFV<-round(out_treelist$BFV,0)
out_treelist$Thin_exp<-round(out_treelist$Thin_exp,3)
out_treelist$folret<-round(out_treelist$folret,2)
out_treelist$ba<-round((out_treelist$Dbh**2*0.005454154*out_treelist$Expan),1)#calculate per acre basal area per tree
out_treelist$CFVpa<-round((out_treelist$Expan*out_treelist$CFV),0)#calculate per acre cubic volume per tree
out_treelist$BFVpa<-round((out_treelist$Expan*out_treelist$BFV),0)#calculate per acre board foot volume per tree
out_treelist$tCFVpa<-round((out_treelist$tCFV*out_treelist$Thin_exp),0)#calculate thinned per acre cubic volume per tree
out_treelist$tBFVpa<-round((out_treelist$tBFV*out_treelist$Thin_exp),0)#calculate thinned per acre board foot volume per tree
out_treelist$Thin_exp[is.na(out_treelist$Thin_exp)]<-0 #convert "NA" to zero


#compute stand-level variables for output
stcfv<-aggregate(list(CFV=out_treelist$CFVpa), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),sum)
stbfv<-aggregate(list(BFV=out_treelist$BFVpa), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin ,Unit=out_treelist$Unit),sum)
sttpa<-aggregate(list(tpa=out_treelist$Expan), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),sum)
stcr<-aggregate(list(CR=out_treelist$CR), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),mean)
sttcfv<-aggregate(list(tCFV=out_treelist$tCFVpa), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),sum)
sttbfv<-aggregate(list(tBFV=out_treelist$tBFVpa), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),sum)
stba<-aggregate(list(BA=out_treelist$ba), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),sum)
stcycle<-aggregate(list(Cycle=out_treelist$Cycle), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),sum)
ststage<-aggregate(list(Stage=out_treelist$Stage), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),mean)
stfolret<-aggregate(list(Stage=out_treelist$folret), list(Cycle=out_treelist$Cycle, Thin=out_treelist$Thin, Unit=out_treelist$Unit),mean)
stqmd<-round((((stba[,4]/sttpa[,4])/3.1416)**0.5)*24,1)#determine QMD by period
stht40<-aggregate(list(ht40=out_treelist$Ht40), list(cycle=out_treelist$Cycle, thin=out_treelist$Thin, unit=out_treelist$Unit),mean)
strd<-round((sttpa[,4]*(stqmd/10)**1.605)/406,3) #determine stand RD by period
sttpa<-round(sttpa,1)#round stand-level TPA
stht40<-round(stht40[,4],2) #round average dominant height
stcr<-round(stcr,2) #round average CR

sttcfv$tCFV[is.na(sttcfv$tCFV)]<-0 #convert "NA" to zero
sttbfv$tBFV[is.na(sttbfv$tBFV)]<-0 #convert "NA" to zero


out_standlist<-cbind(stcycle[,3],stcycle[,1],ststage[,4],sttpa[,4],stba[,4],stqmd,stht40,stcr[,4],strd,stcfv[,4],stbfv[,4],stcycle[,2],sttcfv[,4],sttbfv[,4],round(stfolret[,4],2))
colnames(out_standlist) <- c("Unit","Cycle","Stage","TPA","BA","QMD","Ht40","avCR","RD","CFV","BFV","Thin","thin_CFV","thin_BFV","folret") 
out_standlist<-out_standlist[order(out_standlist[,1],out_standlist[,2],-out_standlist[,4]),]

tr_out<-max(out_treelist$out_tr)

out_standlist<-subset(out_standlist, select=-c(Thin))
out_treelist<-out_treelist[order(out_treelist$Unit,out_treelist$Cycle,out_treelist$Thin,out_treelist$Tree),]
out_treelist<-subset(out_treelist, select=-c(Ht40,out_tr,merch_ht,ba,CFVpa,BFVpa,tCFVpa,tBFVpa))

##########################################################################################################
##########################################################################################################
